@@ -12,6 +12,7 @@ import {
   REGISTRATION_STATUSES,
   SUGGESTER_ENDPOINTS,
   SURVEY_TYPES,
+  TRANSLATABLE_REFERENCE_TYPES,
   VISIBILITY_STATUSES,
   OTHER_REFERENCE_TYPES,
   DEFAULT_LANGUAGE_CODE,
@@ -19,7 +20,19 @@ import {
 import { v4 as uuidv4 } from 'uuid';
 import { customAlphabet } from 'nanoid';
 
-const DEFAULT_LIMIT = 25;
+export const suggestions = express.Router();
+
+const defaultLimit = 25;
+
+const defaultMapper = ({ name, code, id }) => ({ name, code, id });
+
+const replaceDataLabelWithTranslation = (item) => {
+  if (item.dataValues.translated_name) {
+    item.setDataValue('name', item.dataValues.translated_name);
+  }
+  return item;
+};
+
 const ENDPOINT_TO_DATA_TYPE = {
   // Special cases where the endpoint name doesn't match the dataType
   ['facilityLocationGroup']: OTHER_REFERENCE_TYPES.LOCATION_GROUP,
@@ -29,32 +42,25 @@ const ENDPOINT_TO_DATA_TYPE = {
   ['invoiceProducts']: OTHER_REFERENCE_TYPES.INVOICE_PRODUCT,
 };
 const getDataType = (endpoint) => ENDPOINT_TO_DATA_TYPE[endpoint] || endpoint;
-// The string_id for the translated_strings table is a concatenation of this prefix
-// and the id of the record so we need to construct it for the translation attribute
-const getTranslationPrefix = (endpoint) =>
-  `${REFERENCE_DATA_TRANSLATION_PREFIX}.${getDataType(endpoint)}.`;
+const getTranslationPrefix = (dataType) => `${REFERENCE_DATA_TRANSLATION_PREFIX}.${dataType}.`;
 
-// Helper function to generate the translation subquery
-const getTranslationSubquery = (endpoint, modelName) => `(
-  SELECT "text" 
-  FROM "translated_strings" 
-  WHERE "language" = $language
-  AND "string_id" = '${getTranslationPrefix(endpoint)}' || "${modelName}"."id"
-  LIMIT 1
-)`;
-
-// Get the translation label for the record, otherwise the get the untranslated searchColumn
-const translationCoalesce = (endpoint, modelName, searchColumn) =>
-  `COALESCE(${getTranslationSubquery(endpoint, modelName)}, "${modelName}"."${searchColumn}")`;
-const translationCoalesceLiteral = (endpoint, modelName, searchColumn) =>
-  Sequelize.literal(translationCoalesce(endpoint, modelName, searchColumn));
-
-// Overwrite the default search column with translation if it exists
-const getTranslationAttributes = (endpoint, modelName, searchColumn = 'name') => ({
-  include: [[translationCoalesceLiteral(endpoint, modelName, searchColumn), searchColumn]],
-});
-
-export const suggestions = express.Router();
+const getTranslationAttributes = (endpoint, modelName) => {
+  const dataType = getDataType(endpoint);
+  return {
+    include: [
+      [
+        Sequelize.literal(`(
+        SELECT "text" 
+        FROM "translated_strings" 
+        WHERE "language" = :language
+        AND "string_id" = '${getTranslationPrefix(dataType)}' || "${modelName}"."id"
+        LIMIT 1
+      )`),
+        'translated_name',
+      ],
+    ],
+  };
+};
 
 function createSuggesterRoute(
   endpoint,
@@ -73,20 +79,32 @@ function createSuggesterRoute(
 
       const searchQuery = (query.q || '').trim().toLowerCase();
       const positionQuery = literal(
-        `POSITION(LOWER($positionMatch) in LOWER(${translationCoalesce(endpoint, modelName, searchColumn)})) > 1`,
+        `POSITION(LOWER(:positionMatch) in LOWER(${`"${modelName}"."${searchColumn}"`})) > 1`,
       );
+      const dataType = getDataType(endpoint);
+      const translationPrefix = getTranslationPrefix(dataType);
 
-      // We supply the searchQuery to both the whereBuilder and the bind so that we can
-      // either use the bind key in SQL or in the whereBuilder directly using sequelize
-      const where = whereBuilder({
-        search: `%${searchQuery}%`,
-        query,
-        req,
-        endpoint,
-        modelName,
-        searchColumn,
+      const isTranslatable = TRANSLATABLE_REFERENCE_TYPES.includes(dataType);
+      const hasTranslations = await models.TranslatedString.count({
+        where: {
+          language,
+          stringId: {
+            [Op.startsWith]: translationPrefix,
+          },
+        },
       });
 
+      const where =
+        isTranslatable && hasTranslations
+          ? Sequelize.literal(`EXISTS (
+            SELECT 1 
+            FROM translated_strings 
+            WHERE language = :language
+            AND string_id = '${translationPrefix}' || "${modelName}"."id"
+            AND text ILIKE :searchQuery
+          )`)
+          : whereBuilder(`%${searchQuery}%`, query, req);
+
       if (endpoint === 'location' && query.locationGroupId) {
         where.locationGroupId = query.locationGroupId;
       }
@@ -97,28 +115,25 @@ function createSuggesterRoute(
       const results = await model.findAll({
         where,
         include,
-        attributes: getTranslationAttributes(endpoint, modelName, searchColumn),
+        attributes: getTranslationAttributes(endpoint, modelName),
         order: [
-          // TODO: This is a hack to avoid ambiguous column references when we have includes
-          // need to either fix this or enforce custom orderBuilder
           ...(order ? [order] : []),
-          ...(include
-            ? []
-            : [
-                positionQuery,
-                [translationCoalesceLiteral(endpoint, modelName, searchColumn), 'ASC'],
-              ]),
+          positionQuery,
+          [Sequelize.literal(`"${modelName}"."${searchColumn}"`), 'ASC'],
         ],
-        bind: {
+        replacements: {
           positionMatch: searchQuery,
           language,
           searchQuery: `%${searchQuery}%`,
           ...extraReplacementsBuilder(query),
         },
-        limit: DEFAULT_LIMIT,
+        limit: defaultLimit,
       });
+
+      const translatedData = results.map(replaceDataLabelWithTranslation);
+
       // Allow for async mapping functions (currently only used by location suggester)
-      res.send(await Promise.all(results.map(mapper)));
+      res.send(await Promise.all(translatedData.map(mapper)));
     }),
   );
 }
@@ -126,7 +141,7 @@ function createSuggesterRoute(
 // this exists so a control can look up the associated information of a given suggester endpoint
 // when it's already been given an id so that it's guaranteed to have the same structure as the
 // options endpoint
-function createSuggesterLookupRoute(endpoint, modelName, { mapper, searchColumn }) {
+function createSuggesterLookupRoute(endpoint, modelName, { mapper }) {
   suggestions.get(
     `/${endpoint}/:id`,
     asyncHandler(async (req, res) => {
@@ -139,15 +154,19 @@ function createSuggesterLookupRoute(endpoint, modelName, { mapper, searchColumn
 
       const record = await models[modelName].findOne({
         where: { id: params.id },
-        bind: {
+        attributes: getTranslationAttributes(endpoint, modelName),
+        replacements: {
           language,
         },
-        attributes: getTranslationAttributes(endpoint, modelName, searchColumn),
       });
 
       if (!record) throw new NotFoundError();
 
-      res.send(await mapper(record));
+      req.checkPermission('read', record);
+
+      const translatedRecord = replaceDataLabelWithTranslation(record);
+
+      res.send(await mapper(translatedRecord));
     }),
   );
 }
@@ -166,20 +185,23 @@ function createAllRecordsRoute(
       const { language = DEFAULT_LANGUAGE_CODE } = query;
 
       const model = models[modelName];
-      const where = whereBuilder({ search: '%', query, req, endpoint, modelName, searchColumn });
+
+      const where = whereBuilder('%', query, req);
 
       const results = await model.findAll({
         where,
-        order: [[translationCoalesceLiteral(endpoint, modelName, searchColumn), 'ASC']],
-        attributes: getTranslationAttributes(endpoint, modelName, searchColumn),
-        bind: {
+        attributes: getTranslationAttributes(endpoint, modelName),
+        order: [[Sequelize.literal(searchColumn), 'ASC']],
+        replacements: {
           language,
-          searchQuery: '%',
           ...extraReplacementsBuilder(query),
         },
       });
+
+      const translatedResults = results.map(replaceDataLabelWithTranslation);
+
       // Allow for async mapping functions (currently only used by location suggester)
-      res.send(await Promise.all(results.map(mapper)));
+      res.send(await Promise.all(translatedResults.map(mapper)));
     }),
   );
 }
@@ -206,24 +228,6 @@ function createSuggesterCreateRoute(
   );
 }
 
-// Search against the translation if it exists, otherwise search against the searchColumn
-const getTranslationWhereLiteral = (endpoint, modelName, searchColumn) => {
-  return Sequelize.literal(
-    `${translationCoalesce(endpoint, modelName, searchColumn)} ILIKE $searchQuery`,
-  );
-};
-
-const DEFAULT_WHERE_BUILDER = ({ endpoint, modelName, searchColumn = 'name' }) => ({
-  [Op.or]: [getTranslationWhereLiteral(endpoint, modelName, searchColumn)],
-  ...VISIBILITY_CRITERIA,
-});
-
-const DEFAULT_MAPPER = ({ name, code, id }) => ({
-  name,
-  code,
-  id,
-});
-
 // Add a new suggester for a particular model at the given endpoint.
 // Records will be filtered based on the whereSql parameter. The user's search term
 // will be passed to the sql query as ":search" - see the existing suggestion
@@ -236,7 +240,7 @@ function createSuggester(
   allowCreatingNewSuggestion,
 ) {
   const options = {
-    mapper: DEFAULT_MAPPER,
+    mapper: defaultMapper,
     searchColumn: 'name',
     extraReplacementsBuilder: () => {},
     ...optionOverrides,
@@ -287,9 +291,10 @@ const referenceDataBodyBuilder = ({ type, name }) => {
 createSuggester(
   'multiReferenceData',
   'ReferenceData',
-  ({ endpoint, modelName, query: { types } }) => ({
-    ...DEFAULT_WHERE_BUILDER({ endpoint, modelName }),
+  (search, { types }) => ({
     type: { [Op.in]: types },
+    name: { [Op.iLike]: search },
+    ...VISIBILITY_CRITERIA,
   }),
   {
     includeBuilder: (req) => {
@@ -332,7 +337,7 @@ createSuggester(
       if (!types?.length) return;
 
       const caseStatement = types
-        .map((_, index) => `WHEN $type${index} THEN ${index + 1}`)
+        .map((_, index) => `WHEN :type${index} THEN ${index + 1}`)
         .join(' ');
 
       return [
@@ -361,9 +366,10 @@ REFERENCE_TYPE_VALUES.forEach((typeName) => {
   createSuggester(
     typeName,
     'ReferenceData',
-    ({ endpoint, modelName }) => ({
-      ...DEFAULT_WHERE_BUILDER({ endpoint, modelName }),
+    (search) => ({
+      name: { [Op.iLike]: search },
       type: typeName,
+      ...VISIBILITY_CRITERIA,
     }),
     {
       includeBuilder: (req) => {
@@ -377,7 +383,6 @@ REFERENCE_TYPE_VALUES.forEach((typeName) => {
           model: ReferenceData,
           as: 'parent',
           required: true,
-          attributes: ['id'],
           through: {
             attributes: [],
             where: {
@@ -396,16 +401,16 @@ REFERENCE_TYPE_VALUES.forEach((typeName) => {
 });
 
 createSuggester('labTestType', 'LabTestType', () => VISIBILITY_CRITERIA, {
-  mapper: ({ name, code, id, labTestCategoryId }) => ({
-    name,
-    code,
-    id,
-    labTestCategoryId,
-  }),
+  mapper: ({ name, code, id, labTestCategoryId }) => ({ name, code, id, labTestCategoryId }),
 });
 
-const filterByFacilityWhereBuilder = ({ query, modelName, endpoint }) => {
-  const baseWhere = DEFAULT_WHERE_BUILDER({ endpoint, modelName });
+const DEFAULT_WHERE_BUILDER = (search) => ({
+  name: { [Op.iLike]: search },
+  ...VISIBILITY_CRITERIA,
+});
+
+const filterByFacilityWhereBuilder = (search, query) => {
+  const baseWhere = DEFAULT_WHERE_BUILDER(search);
   // Parameters are passed as strings, so we need to check for 'true'
   const shouldFilterByFacility =
     query.filterByFacility === 'true' || query.filterByFacility === true;
@@ -426,9 +431,9 @@ const createNameSuggester = (
   options,
 ) =>
   createSuggester(endpoint, modelName, whereBuilderFn, {
-    mapper: ({ name, id }) => ({
-      name,
+    mapper: ({ id, name }) => ({
       id,
+      name,
     }),
     ...options,
   });
@@ -441,8 +446,8 @@ createSuggester(
   'location',
   'Location',
   // Allow filtering by parent location group
-  ({ endpoint, modelName, query }) => {
-    const baseWhere = filterByFacilityWhereBuilder({ endpoint, modelName, query });
+  (search, query) => {
+    const baseWhere = filterByFacilityWhereBuilder(search, query);
 
     const { ...filters } = query;
     delete filters.q;
@@ -465,7 +470,7 @@ createSuggester(
       const lg = await location.getLocationGroup();
       const locationGroup = lg && { name: lg.name, code: lg.code, id: lg.id };
       return {
-        name: name,
+        name,
         code,
         maxOccupancy,
         id,
@@ -480,25 +485,17 @@ createSuggester(
 createNameSuggester('locationGroup', 'LocationGroup', filterByFacilityWhereBuilder);
 
 // Location groups filtered by facility. Used in the survey form autocomplete
-createNameSuggester('facilityLocationGroup', 'LocationGroup', ({ endpoint, modelName, query }) =>
-  filterByFacilityWhereBuilder({
-    endpoint,
-    modelName,
-    query: { ...query, filterByFacility: true },
-  }),
+createNameSuggester('facilityLocationGroup', 'LocationGroup', (search, query) =>
+  filterByFacilityWhereBuilder(search, { ...query, filterByFacility: true }),
 );
 
 // Location groups filtered by isBookable. Used in location bookings view
-createNameSuggester('bookableLocationGroup', 'LocationGroup', ({ endpoint, modelName, query }) => ({
-  ...filterByFacilityWhereBuilder({
-    endpoint,
-    modelName,
-    query: { ...query, filterByFacility: true },
-  }),
+createNameSuggester('bookableLocationGroup', 'LocationGroup', (search, query) => ({
+  ...filterByFacilityWhereBuilder(search, { ...query, filterByFacility: true }),
   isBookable: true,
 }));
 
-createNameSuggester('survey', 'Survey', ({ search, query: { programId } }) => ({
+createNameSuggester('survey', 'Survey', (search, { programId }) => ({
   name: { [Op.iLike]: search },
   ...(programId ? { programId } : programId),
   surveyType: {
@@ -509,9 +506,10 @@ createNameSuggester('survey', 'Survey', ({ search, query: { programId } }) => ({
 createSuggester(
   'invoiceProducts',
   'InvoiceProduct',
-  ({ endpoint, modelName }) => ({
-    ...DEFAULT_WHERE_BUILDER({ endpoint, modelName }),
+  (search) => ({
+    name: { [Op.iLike]: search },
     '$referenceData.type$': REFERENCE_TYPES.ADDITIONAL_INVOICE_PRODUCT,
+    ...VISIBILITY_CRITERIA,
   }),
   {
     mapper: (product) => {
@@ -533,7 +531,7 @@ createSuggester(
 createSuggester(
   'practitioner',
   'User',
-  ({ search }) => ({
+  (search) => ({
     displayName: { [Op.iLike]: search },
     ...VISIBILITY_CRITERIA,
   }),
@@ -558,7 +556,7 @@ const trimAndConcat = (col1, col2) =>
 createSuggester(
   'patient',
   'Patient',
-  ({ search }) => ({
+  (search) => ({
     [Op.or]: [
       Sequelize.where(trimAndConcat('first_name', 'last_name'), { [Op.iLike]: search }),
       Sequelize.where(trimAndConcat('last_name', 'first_name'), { [Op.iLike]: search }),
@@ -585,8 +583,8 @@ createSuggester(
   },
 );
 
-createSuggester('nonSensitiveLabTestCategory', 'ReferenceData', ({ endpoint, modelName }) => {
-  const baseWhere = DEFAULT_WHERE_BUILDER({ endpoint, modelName });
+createSuggester('nonSensitiveLabTestCategory', 'ReferenceData', (search) => {
+  const baseWhere = DEFAULT_WHERE_BUILDER(search);
   return {
     ...baseWhere,
     type: REFERENCE_TYPES.LAB_TEST_CATEGORY,
@@ -602,8 +600,8 @@ createSuggester('nonSensitiveLabTestCategory', 'ReferenceData', ({ endpoint, mod
   };
 });
 
-createSuggester('sensitiveLabTestCategory', 'ReferenceData', ({ endpoint, modelName }) => {
-  const baseWhere = DEFAULT_WHERE_BUILDER({ endpoint, modelName });
+createSuggester('sensitiveLabTestCategory', 'ReferenceData', (search) => {
+  const baseWhere = DEFAULT_WHERE_BUILDER(search);
   return {
     ...baseWhere,
     type: REFERENCE_TYPES.LAB_TEST_CATEGORY,
@@ -623,8 +621,8 @@ createSuggester('sensitiveLabTestCategory', 'ReferenceData', ({ endpoint, modelN
 createSuggester(
   'patientLabTestCategories',
   'ReferenceData',
-  ({ endpoint, modelName, query, req }) => {
-    const baseWhere = DEFAULT_WHERE_BUILDER({ endpoint, modelName });
+  (search, query, req) => {
+    const baseWhere = DEFAULT_WHERE_BUILDER(search);
 
     if (!query.patientId) {
       return { ...baseWhere, type: REFERENCE_TYPES.LAB_TEST_CATEGORY };
@@ -637,8 +635,8 @@ createSuggester(
           FROM lab_requests
           INNER JOIN
             encounters ON encounters.id = lab_requests.encounter_id
-          WHERE lab_requests.status = $lab_request_status
-            AND encounters.patient_id = $patient_id
+          WHERE lab_requests.status = :lab_request_status
+            AND encounters.patient_id = :patient_id
             AND encounters.deleted_at is null
             AND lab_requests.deleted_at is null
         )`,
@@ -656,8 +654,8 @@ createSuggester(
             ON (lab_requests.id = lab_tests.lab_request_id)
           INNER JOIN lab_test_types
             ON (lab_test_types.id = lab_tests.lab_test_type_id)
-          WHERE lab_requests.status = $lab_request_status
-            AND encounters.patient_id = $patient_id
+          WHERE lab_requests.status = :lab_request_status
+            AND encounters.patient_id = :patient_id
             AND encounters.deleted_at is null
             AND lab_requests.deleted_at is null
             AND lab_test_types.is_sensitive IS FALSE
@@ -683,8 +681,8 @@ createSuggester(
 createSuggester(
   'patientLabTestPanelTypes',
   'LabTestPanel',
-  ({ endpoint, modelName, query }) => {
-    const baseWhere = DEFAULT_WHERE_BUILDER({ endpoint, modelName });
+  (search, query) => {
+    const baseWhere = DEFAULT_WHERE_BUILDER(search);
 
     if (!query.patientId) {
       return baseWhere;
@@ -705,8 +703,8 @@ createSuggester(
             lab_requests ON lab_requests.id = lab_tests.lab_request_id
           INNER JOIN
             encounters ON encounters.id = lab_requests.encounter_id
-          WHERE lab_requests.status = $lab_request_status
-            AND encounters.patient_id = $patient_id
+          WHERE lab_requests.status = :lab_request_status
+            AND encounters.patient_id = :patient_id
             AND encounters.deleted_at is null
             AND lab_requests.deleted_at is null
         )`,
@@ -725,8 +723,8 @@ createSuggester(
 createNameSuggester(
   'programRegistryClinicalStatus',
   'ProgramRegistryClinicalStatus',
-  ({ endpoint, modelName, query: { programRegistryId } }) => ({
-    ...DEFAULT_WHERE_BUILDER({ endpoint, modelName }),
+  (search, { programRegistryId }) => ({
+    ...DEFAULT_WHERE_BUILDER(search),
     ...(programRegistryId ? { programRegistryId } : {}),
   }),
 );
@@ -734,8 +732,8 @@ createNameSuggester(
 createSuggester(
   'programRegistry',
   'ProgramRegistry',
-  ({ endpoint, modelName, query }) => {
-    const baseWhere = DEFAULT_WHERE_BUILDER({ endpoint, modelName });
+  (search, query) => {
+    const baseWhere = DEFAULT_WHERE_BUILDER(search);
     if (!query.patientId) {
       return baseWhere;
     }
@@ -751,7 +749,7 @@ createSuggester(
           INNER JOIN patient_program_registrations ppr
           ON ppr.program_registry_id = pr.id
           WHERE
-            ppr.patient_id = $patient_id
+            ppr.patient_id = :patient_id
           AND
             ppr.registration_status != '${REGISTRATION_STATUSES.RECORDED_IN_ERROR}'
         )`,
@@ -769,8 +767,8 @@ createSuggester(
 createNameSuggester(
   'programRegistryClinicalStatus',
   'ProgramRegistryClinicalStatus',
-  ({ endpoint, modelName, query: { programRegistryId } }) => ({
-    ...DEFAULT_WHERE_BUILDER({ endpoint, modelName }),
+  (search, { programRegistryId }) => ({
+    ...DEFAULT_WHERE_BUILDER(search),
     ...(programRegistryId ? { programRegistryId } : {}),
   }),
 );
@@ -778,8 +776,8 @@ createNameSuggester(
 createNameSuggester(
   'programRegistryCondition',
   'ProgramRegistryCondition',
-  ({ endpoint, modelName, query: { programRegistryId } }) => ({
-    ...DEFAULT_WHERE_BUILDER({ endpoint, modelName }),
+  (search, { programRegistryId }) => ({
+    ...DEFAULT_WHERE_BUILDER(search),
     ...(programRegistryId ? { programRegistryId } : {}),
   }),
 );
@@ -787,8 +785,8 @@ createNameSuggester(
 createNameSuggester(
   'programRegistry',
   'ProgramRegistry',
-  ({ endpoint, modelName, query }) => {
-    const baseWhere = DEFAULT_WHERE_BUILDER({ endpoint, modelName });
+  (search, query) => {
+    const baseWhere = DEFAULT_WHERE_BUILDER(search);
     if (!query.patientId) {
       return baseWhere;
     }
@@ -804,7 +802,7 @@ createNameSuggester(
           INNER JOIN patient_program_registrations ppr
           ON ppr.program_registry_id = pr.id
           WHERE
-            ppr.patient_id = $patient_id
+            ppr.patient_id = :patient_id
           AND
             ppr.registration_status = '${REGISTRATION_STATUSES.ACTIVE}'
         )`,
@@ -822,8 +820,8 @@ createNameSuggester(
 // TODO: Use generic LabTest permissions for this suggester
 createNameSuggester('labTestPanel', 'LabTestPanel');
 
-createNameSuggester('template', 'Template', ({ endpoint, modelName, query }) => {
-  const baseWhere = DEFAULT_WHERE_BUILDER({ endpoint, modelName });
+createNameSuggester('template', 'Template', (search, query) => {
+  const baseWhere = DEFAULT_WHERE_BUILDER(search);
   const { type } = query;
 
   if (!type) {
